/****************************************************************************
FILE IO Library v2.0

Functions for file manipulation.

NO GLOBALS - ShowError should be the only non-native function being called other than stuff from lib_misc.
if used independently of Renegade, delcare int ShowError(const char* FmtString, ...)
*****************************************************************************/
#include "lib_fileio.h"
#include "lib_misc.h"

/****************************************************************************
File Existance check - return 0 if file does NOT exist, file size if exists.
*****************************************************************************/
u32 FileExists(char *filename)
{
	u32 filesize = 0;
    FILE *f = fopen(filename,"rb");
    if (!(f)) { return 0; }
	fseek(f,0,SEEK_END);
	filesize = ftell(f);
	fseek(f,0,SEEK_SET);
    fclose(f);
    return filesize;
}

/****************************************************************************
LoadStruct -Read a struct that was saved as binary.
*****************************************************************************/
int LoadStruct(VOID *buffer, u32 filesize, char* filename)
{
    FILE *f;
    int i;
	f = fopen(filename,"rb");
	if (!(f)) {
        ShowError("Unable to open file: %s (LoadStruct,1) -- Error %u", filename, GetLastError());
        return 0;
    }
	fseek(f,0,SEEK_SET);
	fread(buffer,1,filesize,f);
	fclose(f);
	return filesize;
}

/****************************************************************************
SaveStruct - Write a struct to file as binary.
*****************************************************************************/
int SaveStruct(VOID *buffer, u32 filesize, char* filename)
{
    FILE *f;
    int i;
	f = fopen(filename,"wb");
	if (!(f)) {
        ShowError("Unable to open/create file: %s (SaveStruct,1) -- Error %u", filename, GetLastError());
        return 0;
    }
	fseek(f,0,SEEK_SET);
	fwrite(buffer,1,filesize,f);
	fclose(f);
	return filesize;
}

/****************************************************************************
CopyBinFile -- UpdateProgress can be a function to update progress bar or NULL
*****************************************************************************/
//int CopyBinFile(char *filename, char *newfilename)
CopyBinFile(char *DestFileName, char *OrigFileName, u32 fStart, u32 fChunkSize, int ByteSwap, int AppendFile, int (*UpdateProgress)(u32, u32, u32))
{
    FILE *OrigFile;
    u32 filesize;
	OrigFile = fopen(OrigFileName,"rb");
	if (!(OrigFile)) {
        ShowError("Unable to open source file: %s (CopyBinFile) -- Error %u", OrigFileName, GetLastError());
        return 0;
	}
	fseek(OrigFile,0,SEEK_END);
	filesize = ftell(OrigFile);
	fseek(OrigFile,fStart,SEEK_SET);
	if (fChunkSize < filesize) { filesize = fChunkSize; }
	FILE *DestFile;
	if (AppendFile) { DestFile = fopen(DestFileName,"ab"); }
	else { DestFile = fopen(DestFileName,"wb"); }
	if (!(DestFile)) {
		ShowError("Unable to open destination file: %s (CopyBinFile) -- Error %u", DestFileName, GetLastError());
		fclose(OrigFile); return 0;
	}
/*
	if (ByteSwap == 0) {
		u8 *buffer = NULL;
    	if (!(buffer = (u8*)malloc(filesize+1))) {
        	ShowError("Unable to allocate buffer memory (CopyBinFile) -- Error %u", GetLastError());
        	fclose(OrigFile); fclose(DestFile); return 0;
    	}
		fread(buffer,1,filesize,OrigFile);
		fwrite(buffer,1,filesize,DestFile);
		free(buffer);
	}
*/
	u32 BuffSize = 0xA00000; //10MB
	u32 BytesRead = 0;
	u8 *buffer = NULL;
//	if (buffer) { free(buffer); buffer = NULL; }
    if (!(buffer = (u8*)malloc(BuffSize))) {
        ShowError("Unable to allocate buffer memory (CopyBinFile) -- Error %u", GetLastError());
        fclose(OrigFile); fclose(DestFile); return 0;
    }
//	do {
	while (BytesRead = fread(buffer,1,BuffSize,OrigFile)) {
//ShowError("loop: %u, %08X", BytesRead, BuffSize);
		if (ferror(OrigFile)) {
			ShowError("Error reading file: %s (CopyBinFile) -- Error %u", OrigFileName, GetLastError());
			break;
		}
//		if ((BytesRead + BuffSize) > filesize) { BuffSize = filesize - BytesRead; }
//		BytesRead += BuffSize;
		if (ByteSwap) {
			SwapBytesArray(buffer, BuffSize, ByteSwap);
//			ShowError("bah: %08X", buffer[4]);
		}
		fwrite(buffer,1,BytesRead,DestFile);
		if (*UpdateProgress != NULL) { UpdateProgress(BytesRead, BuffSize, filesize); }
	} //while (BytesRead < filesize);


	if (buffer) { free(buffer); buffer = NULL; }
	fclose(OrigFile);
	fclose(DestFile);
	return 1;
}

/****************************************************************************
Load File
Loads a file (filename), cuts the header (headerlen) and reads the data into
buffer. if loadheader is true, it also reads the header into the headerdata.
*****************************************************************************/
int LoadFile(u8 **buffer, char* filename, int headerlen, u8 **headerdata, BOOL loadheader, u32 ChunkSize)
{
    FILE *f;
    int i;
    u32 filesize;
	f = fopen(filename,"rb");
	if (!(f)) {
        ShowError("Unable to open file: %s (LoadFile,1) -- Error %u", filename, GetLastError());
        return 0;
    }
	fseek(f,0,SEEK_END);
	filesize = ftell(f);
	fseek(f,0,SEEK_SET);
    if (headerlen > filesize) {
	    ShowError("Header length greater than file size. WTF? Idiot. (LoadFile,1: %s)", filename);
	    fclose(f); return 0;
	}
	//buffer malloc was here
    if ((loadheader) && (headerlen)) {
        if (*headerdata) { free(*headerdata); *headerdata = NULL; }
        if (!(*headerdata = (unsigned char*)malloc(headerlen))) {
            ShowError("Unable to allocate header memory for file: %s (LoadFile, 1) -- Error %u", filename, GetLastError());
            free(*headerdata); *headerdata = NULL;
            free(*buffer); *buffer = NULL;
            fclose(f); return 0;
        }
        fread(*headerdata,1,headerlen,f);
        filesize -= headerlen;
    } else {
        filesize -= headerlen;
        fseek(f,headerlen,SEEK_SET);
    }

    /***read only part of the file?***/
    if (ChunkSize < filesize) { filesize = ChunkSize; }

    if (*buffer) { free(*buffer); *buffer = NULL; }
    if (!(*buffer = (unsigned char*)malloc(filesize+1))) {
        ShowError("Unable to allocate buffer memory for file: %s (LoadFile, 1) -- Error %u", filename, GetLastError());
        fclose(f); return 0;
    }

	fread(*buffer,1,filesize,f);
//	fread(*buffer,1, (ChunkSize > filesize) ? filesize : ChunkSize ,f);
	fclose(f);
	return filesize;
}


/****************************************************************************
SaveFile - Set headlen to -1 for append
*****************************************************************************/
int SaveFile(u8 *buffer, u32 filesize, char* filename, int headerlen, VOID *headerdata)
{
    FILE *f;
    int i;
    if (headerlen == -1) { f = fopen(filename,"ab"); }
	else { f = fopen(filename,"wb"); }
	if (!(f)) {
        ShowError("Unable to open/create file: %s (SaveFile,1) -- Error %u", filename, GetLastError());
        return 0;
    }
	fseek(f,0,SEEK_SET);
	if (headerlen > 0) { fwrite(headerdata,1,headerlen,f); }
	fwrite(buffer,1,filesize,f);
	fclose(f);
	return filesize;
}
